from upload_machine.utils.pathinfo.pathinfo import findnum
import os
from loguru import logger
import time
import requests
import re
import json
import sys
from upload_machine.utils.img_upload.imgupload import img_upload
from upload_machine.utils.edittorrent.edittorrent import *
from shutil import move
from doubaninfo.doubaninfo import getdoubaninfo

def deletetorrent(delpath=''):
    if delpath=='':
        delpath = os.path.dirname(os.path.abspath(__file__))
    logger.info('正在清除此文件夹下所有种子文件:'+delpath)
    ls = os.listdir(delpath)
    for i in ls:
        c_path = os.path.join(delpath, i)
        if (os.path.isfile(c_path)) and (os.path.splitext(i)[1].lower()== ('.torrent')  or i.startswith('.')):
            logger.info('正在删除文件:'+c_path+'...')
            os.remove(c_path)

def changename(path):
    if not 'win' in sys.platform:
        return path
    else:
        return '/cygdrive/'+path.replace('\\','/').replace(':','')
def mktorrent_win_temp(filepath,torrentname,tracker="https://announce.leaguehd.com/announce.php"):
    cwd=os.getcwd()
    newcwd=os.path.dirname(filepath)
    os.chdir(newcwd)
    if os.path.isdir(filepath):
        logger.info('检测到路径制种，将先删除掉路径里面所有种子文件(torrent后缀)以及隐藏文件（.开头的文件）...')
        deletetorrent(filepath) 
    logger.info('即将开始制作种子...')
    if os.path.exists(torrentname):
        logger.info('已存在种子文件，正在删除'+torrentname)
        try:
            os.remove(torrentname)
        except Exception as r:
            logger.error('删除种子发生错误: %s' %(r))
    logger.info('正在制作种子:'+filepath)
    order='mktorrent -v -p -f -l 24 -a '+tracker+' -o \"'+changename(torrentname)+ '\" \"'+os.path.basename(filepath)+'\"'+''
    logger.info(order)
    os.system(order)
    logger.info('已完成制作种子'+torrentname)
    os.chdir(cwd)

def mktorrent_win(filepath,torrentname,tracker="https://announce.leaguehd.com/announce.php"):
    cwd=os.getcwd()
    newcwd=os.path.dirname(filepath)
    os.chdir(newcwd)
    if os.path.isdir(filepath):
        logger.info('检测到路径制种，将先删除掉路径里面所有种子文件(torrent后缀)以及隐藏文件（.开头的文件）...')
        deletetorrent(filepath) 
    logger.info('即将开始制作种子...')
    if os.path.exists(torrentname):
        logger.info('已存在种子文件，正在删除'+torrentname)
        try:
            os.remove(torrentname)
        except Exception as r:
            logger.error('删除种子发生错误: %s' %(r))

    if os.path.isdir(filepath):
        new_filepath=os.path.join(os.path.dirname(filepath),'AAABBBCCC')
        os.rename(filepath,new_filepath)
        logger.info('正在制作种子:'+filepath)
        order='mktorrent -v -p -l 24 -c "Made by Auto_Upload" -a '+tracker+' -o \"'+changename(torrentname)+ '\" \"'+os.path.basename(new_filepath)+'\"'+''
        #logger.info(order)
        
        trytime=0
        filesize=0
        res=''
        while filesize==0:
            
            trytime=trytime+1
            if trytime>10:
                logger.error('制作种子失败')
                break
            if trytime>1:
                logger.warning('制作种子失败,失败原因:'+str(res))
                logger.info(order)
            logger.info('第'+str(trytime)+'次制作种子:')

            #os.system(order)
            if os.path.exists(torrentname):
                logger.info('已存在种子文件，正在删除'+torrentname)
                try:
                    os.remove(torrentname)
                except Exception as r:
                    logger.error('删除种子发生错误: %s' %(r))
            res=os.popen(order)
            res=res.buffer.read().decode('utf-8')
            if os.path.exists(torrentname):
                filesize=os.path.getsize(torrentname)
            else:
                filesize=0

        os.rename(new_filepath,filepath)
        t=Torrent()
        t.load(torrentname)
        t.data[b"info"][b"name"]=str2bytes(os.path.basename(filepath))
        t.dump(torrentname)
    else:
        new_filepath=os.path.join(os.path.dirname(filepath),'AAABBBCCC'+os.path.splitext(filepath)[-1])
        os.rename(filepath,new_filepath)
        logger.info('正在制作种子:'+filepath)
        order='mktorrent -v -p -l 24 -c "Made by Auto_Upload" -a '+tracker+' -o \"'+changename(torrentname)+ '\" \"'+os.path.basename(new_filepath)+'\"'+''

        #logger.info(order)

        trytime=0
        filesize=0
        res=''
        while filesize==0:
            trytime=trytime+1
            if trytime>10:
                logger.error('制作种子失败')
                break
            if trytime>1:
                logger.warning('制作种子失败,失败原因:'+str(res))
                if os.path.exists(torrentname):
                    logger.info('已存在种子文件，正在删除'+torrentname)
                    try:
                        os.remove(torrentname)
                    except Exception as r:
                        logger.error('删除种子发生错误: %s' %(r))
                logger.info(order)
            logger.info('第'+str(trytime)+'次制作种子:')

            #os.system(order)
            res=os.popen(order)
            res=res.buffer.read().decode('utf-8')
            filesizetime=0
            while filesize==0:
                filesizetime=filesizetime+1
                if filesizetime>5:
                    break
                if os.path.exists(torrentname):
                    filesize=os.path.getsize(torrentname)
                else:
                    filesize=0
                if filesize==0:
                    time.sleep(1)
            

        os.rename(new_filepath,filepath)
        t=Torrent()
        t.load(torrentname)
        t.data[b"info"][b"name"]=str2bytes(os.path.basename(filepath))
        t.dump(torrentname)



    logger.info('已完成制作种子'+torrentname)
    os.chdir(cwd)

def mktorrent(filepath,torrentname,tracker="https://announce.leaguehd.com/announce.php"):
    if 'win32' in sys.platform:
        mktorrent_win(filepath,torrentname,tracker)
        return 
    if os.path.isdir(filepath):
        logger.info('检测到路径制种，将先删除掉路径里面所有种子文件(torrent后缀)以及隐藏文件（.开头的文件）...')
        deletetorrent(filepath) 
    deletetorrent(os.path.dirname(torrentname))
    logger.info('即将开始制作种子...')
    trytime=0
    while os.path.exists(torrentname):
        trytime=trytime+1
        if trytime>5:
            logger.error('删除种子失败，制作种子失败')
            break
        logger.info('已存在种子文件，正在删除'+torrentname)
        try:
            os.rename(torrentname,torrentname+'temp')
            os.remove(torrentname+'temp')
        except Exception as r:
            logger.warning('删除种子发生错误: %s' %(r))

    logger.info('正在对下面路径制作种子:'+filepath)
    #order='mktorrent -v -p -f -l 24 -c "Made by Auto_Upload" -a '+tracker+' -o \"'+torrentname+ '\" \"'+filepath+'\"'+' > /dev/null'
    order='mktorrent -v -p -l 24 -c "Made by Auto_Upload" -a '+tracker+' -o \"'+torrentname+ '\" \"'+filepath+'\"'
        
    #logger.info(order)
    trytime=0
    filesize=0
    while filesize==0:
        trytime=trytime+1
        if trytime>10:
            logger.error('制作种子失败')
            break
        if trytime>1:
                logger.warning('制作种子失败,失败原因:'+str(res))
                logger.info(order)
        logger.info('第'+str(trytime)+'次制作种子:')
        #os.system(order)
        if os.path.exists(torrentname):
            logger.info('已存在种子文件，正在删除'+torrentname)
            try:
                os.remove(torrentname)
            except Exception as r:
                logger.error('删除种子发生错误: %s' %(r))
        res=os.popen(order)
        res=res.buffer.read().decode('utf-8')
        filesizetime=0
        while filesize==0:
            filesizetime=filesizetime+1
            if filesizetime>5:
                break
            if os.path.exists(torrentname):
                filesize=os.path.getsize(torrentname)
            else:
                filesize=0
            if filesize==0:
                time.sleep(1)
        

    logger.info('已完成制作种子'+torrentname)


def get_video_duration(video_path: str):
    ext = os.path.splitext(video_path)[-1]
    if ext != '.mp4' and ext != '.avi' and ext != '.flv'and ext != '.ts'and ext != '.mkv':
        raise Exception('format not support')
    ffprobe_cmd = 'ffprobe -i "'+video_path+'" -show_entries format=duration -v quiet -of csv="p=0"'
    a=os.popen(ffprobe_cmd)
    duration_info = float(a.read())
    #duration_info = float(a.buffer.read().decode('utf-8'))
    return duration_info

def takescreenshot(file,screenshotaddress,screenshotnum):
    '''
    para:
        file:视频文件
        duration:视频文件播放时长
        screenshotaddress:存放截图地址，截图后除了新截图其他文件会被清空
        screenshotnum:截图数量
    '''
    logger.info('正在对视频'+file+'截图'+str(screenshotnum)+'张并存入文件夹'+screenshotaddress)
    i=os.path.basename(file)
    if (os.path.isdir(file)) or (i.startswith('.')) or (not(  os.path.splitext(i)[1].lower()== ('.mp4') or os.path.splitext(i)[1].lower()== ('.mkv')  or os.path.splitext(i)[1].lower()== ('.avi') or os.path.splitext(i)[1].lower()== ('.ts')    )):
        logger.error(file+'非视频文件,无法截图')
        raise ValueError (file+'非视频文件,无法截图')
    duration=get_video_duration(file)
    if abs(duration)<1e-9:
        logger.error(file+'视频文件播放时长过短,无法截图')
        raise ValueError (file+'视频文件播放时长过短,无法截图')
    #清空截图文件夹
    ls = os.listdir(screenshotaddress)
    for i in ls:
        c_path = os.path.join(screenshotaddress, i)
        if not (os.path.isdir(c_path)):
            os.remove(c_path)
    timestep=duration*1.0/(screenshotnum+3)
    firststep=timestep*2
    for i in range (screenshotnum):
        firststep=firststep+timestep
        if 'win32' in sys.platform:
            screenshotstr='ffmpeg -ss '+str(firststep)+' -i "'+file+'" -f image2 -y "'+os.path.join(screenshotaddress,str(i+1)+'.jpg')+'"'
        else:
            screenshotstr='ffmpeg -ss '+str(firststep)+' -i "'+file+'" -f image2 -y "'+os.path.join(screenshotaddress,str(i+1)+'.jpg')+'" &> /dev/null'
        #print(screenshotstr)
        os.system(screenshotstr)
    logger.info('截图完毕')




class mediafile(object):
    def __init__(self,mediapath,pathinfo,basic,imgdata):
        self.mediapath         =mediapath
        self.pathinfo          = pathinfo
        self.downloadpath      =pathinfo.downloadpath
        self.basic             =basic
        #self.address就是要被抓去info的资源文件路径
        #如果mediapath是文件夹，self.address就选里面最大的视频文件
        if os.path.isdir(self.mediapath):
            self.isdir=True
            maxsize=0
            ls = os.listdir(self.mediapath)
            for i in ls:
                c_path=os.path.join(self.mediapath, i)
                if (os.path.isdir(c_path)) or (i.startswith('.')) or (not(  os.path.splitext(i)[1].lower()== ('.mp4') or os.path.splitext(i)[1].lower()== ('.mkv')  or os.path.splitext(i)[1].lower()== ('.avi') or os.path.splitext(i)[1].lower()== ('.ts')    )):
                    continue
                filesize=os.path.getsize(c_path)
                if filesize>maxsize:
                    self.address=c_path
                    maxsize=filesize

            if 'zeroday_name' in self.pathinfo.infodict and self.pathinfo.infodict['zeroday_name']!=None and self.pathinfo.infodict['zeroday_name']!='':
                zeroday_path=os.path.join(self.pathinfo.path,self.pathinfo.infodict['zeroday_name'])
                if os.path.exists(zeroday_path):
                    ls = os.listdir(zeroday_path)
                    for i in ls:
                        c_path=os.path.join(zeroday_path, i)
                        if (os.path.isdir(c_path)) or (i.startswith('.')) or (not(  os.path.splitext(i)[1].lower()== ('.mp4') or os.path.splitext(i)[1].lower()== ('.mkv')  or os.path.splitext(i)[1].lower()== ('.avi') or os.path.splitext(i)[1].lower()== ('.ts')    )):
                            continue
                        filesize=os.path.getsize(c_path)
                        if filesize>maxsize:
                            self.address=c_path
                            maxsize=filesize
        
        else:
            self.address           = mediapath
            self.isdir=False
            
 
        
        #种子目录
        self.topath            = ''
        self.screenshotaddress = basic['screenshot_path']
        self.screenshotnum     = int(basic['picture_num'])
        self.imgdata           = imgdata
        
        self.path              = os.path.dirname(mediapath)
        
        self.mediatype         = pathinfo.type
        self.doubanurl         = pathinfo.doubanurl
        self.imdburl           = pathinfo.imdb_url
        self.bgmurl            = pathinfo.bgm_url
        self.anidburl          = pathinfo.anidb_url
        self.from_url          = pathinfo.from_url
        self.transfer          = pathinfo.transfer
        self.filename          = os.path.basename(self.address)
        self.episodename       = findnum(self.filename)[0]
        self.episode           = int(self.episodename)

        self.sub               = self.pathinfo.sub
        self.englishname       = self.pathinfo.englishname
        self.chinesename       = self.pathinfo.chinesename

        self.audio_ch=0
        self.audio_jp=0
        self.audio_en=0
        self.audio_yue=0
        self.audio_num=0

        self.text_jp=0
        self.text_sc=0
        self.text_tc=0
        self.text_en=0
 
        if ('anime' in self.mediatype.lower() or 'tv' in self.mediatype.lower() ):
            self.season            = self.pathinfo.season
            self.seasonnum         = self.pathinfo.seasonnum
            self.season_ch         = self.pathinfo.season_ch
            self.complete          = self.pathinfo.complete

        dlgroup           =['NaN-Raws','NaN Raws','NC-Raws','NC Raws','Lilith-Raws','Lilith Raws','ANi','Skymoon-Raws','Skymoon Raws']
        self.type              ='WEBRip'
        self.Video_Format      ='H264'
        
        if 'hdtvrip' in self.filename.lower() or 'hdtv-rip' in self.filename.lower() or 'tv-rip' in self.filename.lower() or 'tvrip' in self.filename.lower():
            self.type='HDTVRip'
        elif 'hdtv' in self.filename.lower():
            self.type='HDTV'
        elif 'bdrip' in self.filename.lower() or 'bd-rip' in self.filename.lower():
            self.type='BDRip'
        elif 'remux' in self.filename.lower():
            self.type='Remux'
        elif 'bluray' in self.filename.lower() or 'blu-ray' in self.filename.lower():
            self.type='Bluray'
        elif 'dvdrip' in self.filename.lower() or 'dvd-rip' in self.filename.lower():
            self.type='DVDRip'
        elif 'dvd' in self.filename.lower() :
            self.type='DVD'
        elif 'webdl' in self.filename.lower() or 'web-dl' in self.filename.lower() :
            self.type='WEB-DL'
        elif 'webrip' in self.filename.lower() or 'web-rip' in self.filename.lower():
            self.type='WEBRip'
        elif self.sub in dlgroup or ('WEB' in self.sub.upper() and not 'WEBRRP' in self.sub.upper() and not '爪爪' in self.pathinfo.exinfo):
            self.type='WEB-DL'

        self.getscreenshot_done=0
        self.getimgurl_done=0
        self.getmediainfo_done=0
        self.getptgen_done=0
        self.mktorrent_done=0
        self.getinfo_done=0


        self.language=''
        self.country=''
        self.year=2022

        #根据文件名判断内嵌字幕信息
        self.sublan=''
        jp=0
        sc=0
        tc=0
        if 'SC' in self.filename.upper() or 'CHS' in self.filename.upper() or 'GB' in self.filename.upper() or '简' in self.filename.upper() or '簡' in self.filename.upper():
            sc=1
        if 'TC' in self.filename.upper() or 'CHT' in self.filename.upper() or 'BIG5' in self.filename.upper() or '繁' in self.filename.upper():
            tc=1
        if 'JP' in self.filename.upper() or 'JAPANESE' in self.filename.upper() or '日' in self.filename.upper():
            jp=1
        
        if jp==0 and sc==0 and tc==1:
            self.sublan='[内嵌繁中]'
        elif jp==0 and sc==1 and tc==0:
            self.sublan='[内嵌简中]'
        elif jp==0 and sc==1 and tc==1:
            self.sublan='[内嵌简繁中字]'
        elif jp==1 and sc==0 and tc==0:
            self.sublan='[内嵌日字]'
        elif jp==1 and sc==0 and tc==1:
            self.sublan='[内嵌繁日双语]'
        elif jp==1 and sc==1 and tc==0:
            self.sublan='[内嵌简日双语]'
        elif jp==1 and sc==1 and tc==1:
            self.sublan='[内嵌简繁日双语]'
        else:
            self.sublan=''
        
        if jp+sc+tc>0:
            logger.info('根据文件名分析，字幕语言为'+self.sublan)
        else:
            logger.warning('无法根据文件名分析出字幕语言信息')



    def getscreenshot(self):
        if self.getscreenshot_done==0:
            takescreenshot(self.address,self.screenshotaddress,self.screenshotnum)
            self.getscreenshot_done=1
    
    def getimgurl(self,server=''):
        if self.getimgurl_done==1:
            return self.screenshoturl
        '''
        0张图的特判
        '''
        if 'picture_num' in self.basic and int(self.basic['picture_num'])==0:
            self.screenshoturl=''
            self.getimgurl_done=1
            return ''
        self.getscreenshot()
        imgpaths=[]
        for i in range (self.screenshotnum):
            imgpaths.append(os.path.join(self.screenshotaddress,str(i+1)+'.jpg'))
        logger.info('正在将'+self.chinesename +'的第'+self.episodename+'集截图上传'+server+'图床,请稍等...')
        res=img_upload(imgdata=self.imgdata,imglist=imgpaths,host=server,form='bbcode')
        if res=='':
            print(self.chinesename+'上传图床失败,请自行上传图床：')
            temp='\n'
            res=''
            while not temp=='':
                temp=input('')
                res=res+temp+'\n'
        logger.info('成功获得图片链接：\n'+res)
        self.screenshoturl=res
        self.getimgurl_done=1
        return self.screenshoturl

    def dealsubtext(self,res):
        jp=0
        sc=0
        tc=0
        en=0
        a=res.split('\n\n')
        for item in a:
            if item.startswith('Text'):
                b=item.split('\n')
                for subitem in b:
                    if subitem.lower().startswith('language') or subitem.lower().startswith('title'):
                        if 'SC' in subitem.upper() or 'CHS' in subitem.upper() or 'GB' in subitem.upper() or '简' in subitem.upper() or '簡' in subitem.upper():
                            sc=1
                        if 'TC' in subitem.upper() or 'CHT' in subitem.upper() or 'BIG5' in subitem.upper() or '繁' in subitem.upper():
                            tc=1
                        if 'JP' in subitem.upper() or 'JA' in subitem.upper() or'JAPANESE' in subitem.upper() or '日' in subitem.upper():
                            jp=1
                        if 'EN' in subitem.upper() or 'ENGLISH' in subitem.upper() or '英' in subitem.upper():
                            en=1
        if en==0 and jp==0 and sc==0 and tc==1:
            self.sublan='[繁体中字]'
        elif en==0 and jp==0 and sc==1 and tc==0:
            self.sublan='[简体中字]'
        elif en==0 and jp==0 and sc==1 and tc==1:
            self.sublan='[简繁中字]'
        elif en==0 and jp==1 and sc==0 and tc==0:
            self.sublan='[日文字幕]'
        elif en==0 and jp==1 and sc==0 and tc==1:
            self.sublan='[繁日双语]'
        elif en==0 and jp==1 and sc==1 and tc==0:
            self.sublan='[简日双语]'
        elif en==0 and jp==1 and sc==1 and tc==1:
            self.sublan='[简繁日双语]'
        elif en==1 and jp==0 and sc==0 and tc==0:
            self.sublan='[英文字幕]'
        elif en==1 and jp==0 and sc==0 and tc==1:
            self.sublan='[繁英双语]'
        elif en==1 and jp==0 and sc==1 and tc==0:
            self.sublan='[简英双语]'
        elif en==1 and jp==0 and sc==1 and tc==1:
            self.sublan='[简繁英双语]'
        elif en==1 and jp==1 and sc==0 and tc==0:
            self.sublan='[日英双语]'
        elif en==1 and jp==1 and sc==0 and tc==1:
            self.sublan='[繁日英三语]'
        elif en==1 and jp==1 and sc==1 and tc==0:
            self.sublan='[简日英三语]'
        elif en==0 and jp==1 and sc==1 and tc==1:
            self.sublan='[简繁日英三语]'

        self.text_jp=jp
        self.text_sc=sc
        self.text_tc=tc
        self.text_en=en

        if jp+sc+tc+en>0:
            logger.info('根据mediainfo分析，字幕语言为'+self.sublan)
        else:
            logger.warning('无法根据mediainfo分析出字幕语言信息')

    def dealaudio(self,res):
        ch=0
        jp=0
        en=0
        a=res.split('\n\n')
        for item in a:
            if item.startswith('Audio'):
                b=item.split('\n')
                for subitem in b:
                    if subitem.lower().startswith('language') or subitem.lower().startswith('title'):

                        if 'CHINESE' in subitem.upper() or '中' in subitem.upper() or '国' in subitem.upper():
                            ch=1
                            #logger.info('根据mediainfo音轨分析，语言为'+self.language)
                        if  'JAPANESE' in subitem.upper() or '日' in subitem.upper():
                            jp=1
                            #logger.info('根据mediainfo音轨分析，语言为'+self.language)
                        if 'ENGLISH' in subitem.upper() or '英' in subitem.upper():
                            en=1
                            #logger.info('根据mediainfo音轨分析，语言为'+self.language)
                        if 'CANTON' in subitem.upper() or '粤' in subitem.upper():
                            yue=1

        '''
        if jp==0 and ch==0 and en==1:
            self.language='英语'
        elif jp==0 and ch==1 and en==0:
            self.language='国语'
        elif jp==0 and ch==1 and en==1:
            self.language='中英双语'
        elif jp==1 and ch==0 and en==0:
            self.language='日语'
        elif jp==1 and ch==0 and en==1:
            self.language='日英双语'
        elif jp==1 and ch==1 and en==0:
            self.language='中日双语'
        elif jp==1 and ch==1 and en==1:
            self.language='中英日三语'
        else:
            self.language=self.language
        '''
        if jp+ch+en+yue>0:
            self.language=''
            if ch==1:
                self.language=self.language+'国'
            if yue==1:
                self.language=self.language+'粤'
            if jp==1:
                self.language=self.language+'日'
            if en==1:
                self.language=self.language+'英'
        self.audio_num=jp+ch+en+yue
        self.audio_ch=ch
        self.audio_jp=jp
        self.audio_en=en
        self.audio_yue=yue

        if self.audio_num=='1':
            self.language=self.language+'语'
        elif self.audio_num=='2':
            self.language=self.language+'双语'
        elif self.audio_num=='3':
            self.language=self.language+'三语'
        elif self.audio_num=='4':
            self.language=self.language+'四语'

        if self.audio_num>0:
            logger.info('根据mediainfo音轨分析，语言为'+self.language)
        else:
            logger.warning('无法根据mediainfo分析出音轨语言信息')

    def getaudio(self):
        ch=0
        jp=0
        en=0
        infolist=self.mediainfo_json['media']['track']
        for item in infolist:
            if not '@type' in item:
                continue
            if item['@type'].lower()=='audio':
                if 'Title' in item :
                    if 'CHINESE' in item['Title'].upper() or '中' in item['Title'].upper() or 'CH' in item['Title'].upper() or 'ZH' in item['Title'].upper() or '国' in item['Title'].upper():
                        ch=1
                    if  'JA' in item['Title'].upper() or 'JP' in item['Title'].upper() or '日' in item['Title'].upper():
                        jp=1
                    if 'EN' in item['Title'].upper() or '英' in item['Title'].upper():
                        en=1
                elif 'Language' in item :
                    if 'CHINESE' in item['Language'].upper() or '中' in item['Language'].upper() or 'CH' in item['Language'].upper() or 'ZH' in item['Language'].upper() or '国' in item['Language'].upper():
                        ch=1
                    if  'JA' in item['Language'].upper() or 'JP' in item['Language'].upper() or '日' in item['Language'].upper():
                        jp=1
                    if 'EN' in item['Language'].upper() or '英' in item['Language'].upper():
                        en=1

        if jp==0 and ch==0 and en==1:
            self.language='英语'
        elif jp==0 and ch==1 and en==0:
            self.language='国语'
        elif jp==0 and ch==1 and en==1:
            self.language='中英双语'
        elif jp==1 and ch==0 and en==0:
            self.language='日语'
        elif jp==1 and ch==0 and en==1:
            self.language='日英双语'
        elif jp==1 and ch==1 and en==0:
            self.language='中日双语'
        elif jp==1 and ch==1 and en==1:
            self.language='中英日三语'
        else:
            self.language=self.language

        self.audio_ch=ch
        self.audio_jp=jp
        self.audio_en=en

        if jp+ch+en>0:
            logger.info('根据mediainfo音轨分析，语言为'+self.language)
        else:
            logger.warning('无法根据mediainfo分析出音轨语言信息')


    def getsubtext(self):
        jp=0
        sc=0
        tc=0
        en=0
        infolist=self.mediainfo_json['media']['track']
        for item in infolist:
            if not '@type' in item:
                continue
            if item['@type'].lower()=='text':
                if 'Title' in item :
                    if 'TC' in item['Title'].upper() or 'CHT' in item['Title'].upper() or 'BIG5' in item['Title'].upper() or '繁' in item['Title'].upper():
                        tc=1
                    elif 'SC' in item['Title'].upper() or 'CHS' in item['Title'].upper() or 'GB' in item['Title'].upper() or '简' in item['Title'].upper() or '簡' in item['Title'].upper() or '中' in item['Title'].upper():
                        sc=1
                    elif 'JP' in item['Title'].upper() or 'JAPANESE' in item['Title'].upper() or '日' in item['Title'].upper():
                        jp=1
                    elif 'EN' in item['Title'].upper() or 'ENGLISH' in item['Title'].upper() or '英' in item['Title'].upper():
                        en=1
                elif 'Language' in item:
                    if 'TC' in item['Language'].upper() or 'CHT' in item['Language'].upper() or 'BIG5' in item['Language'].upper() or '繁' in item['Language'].upper():
                        tc=1
                    elif 'SC' in item['Language'].upper() or 'CHS' in item['Language'].upper() or 'GB' in item['Language'].upper() or '简' in item['Language'].upper() or '簡' in item['Language'].upper() or 'ZH' in item['Language'].upper():
                        sc=1
                    elif 'JP' in item['Language'].upper() or 'JA' in item['Language'].upper() or 'JAPANESE' in item['Language'].upper() or '日' in item['Language'].upper():
                        jp=1
                    elif 'EN' in item['Language'].upper() or 'ENGLISH' in item['Language'].upper() or '英' in item['Language'].upper():
                        en=1
        if en==0 and jp==0 and sc==0 and tc==1:
            self.sublan='[繁体中字]'
        elif en==0 and jp==0 and sc==1 and tc==0:
            self.sublan='[简体中字]'
        elif en==0 and jp==0 and sc==1 and tc==1:
            self.sublan='[简繁中字]'
        elif en==0 and jp==1 and sc==0 and tc==0:
            self.sublan='[日文字幕]'
        elif en==0 and jp==1 and sc==0 and tc==1:
            self.sublan='[繁日双语]'
        elif en==0 and jp==1 and sc==1 and tc==0:
            self.sublan='[简日双语]'
        elif en==0 and jp==1 and sc==1 and tc==1:
            self.sublan='[简繁日双语]'
        elif en==1 and jp==0 and sc==0 and tc==0:
            self.sublan='[英文字幕]'
        elif en==1 and jp==0 and sc==0 and tc==1:
            self.sublan='[繁英双语]'
        elif en==1 and jp==0 and sc==1 and tc==0:
            self.sublan='[简英双语]'
        elif en==1 and jp==0 and sc==1 and tc==1:
            self.sublan='[简繁英双语]'
        elif en==1 and jp==1 and sc==0 and tc==0:
            self.sublan='[日英双语]'
        elif en==1 and jp==1 and sc==0 and tc==1:
            self.sublan='[繁日英三语]'
        elif en==1 and jp==1 and sc==1 and tc==0:
            self.sublan='[简日英三语]'
        elif en==0 and jp==1 and sc==1 and tc==1:
            self.sublan='[简繁日英三语]'

        self.text_jp=jp
        self.text_sc=sc
        self.text_tc=tc
        self.text_en=en

        if jp+sc+tc+en>0:
            logger.info('根据mediainfo分析，字幕语言为'+self.sublan)
        else:
            logger.warning('无法根据mediainfo分析出字幕语言信息')

    def updatemediainfo(self,filepath=''):
        if filepath!='':
            a=os.popen('mediainfo --Inform=file://"'+filepath+'" "'+self.address+'"')
            res=a.buffer.read().decode('utf-8')
        else:
            a=os.popen('mediainfo "'+self.address+'"')
            res=a.buffer.read().decode('utf-8')
            ss=res.split('\n')
            for i in range(len(ss)):
                if ss[i].startswith('Complete name'):
                    ss[i]=':'.join([ss[i].split(':')[0],' '+self.filename])
            res='\n'.join(ss)

        self.mediainfo=res

    def getmediainfo(self):
        if self.getmediainfo_done==1:
            return self.mediainfo
        a=os.popen('mediainfo "'+self.address+'"')
        #res=a.read()
        res=a.buffer.read().decode('utf-8')
        #self.dealsubtext(res)
        #self.dealaudio(res)
        ss=res.split('\n')
        #ss[1]=':'.join([ss[1].split(':')[0],' '+self.filename])
        for i in range(len(ss)):
            if ss[i].startswith('Complete name'):
                ss[i]=':'.join([ss[i].split(':')[0],' '+self.filename])

        self.mediainfo='\n'.join(ss)
        a=os.popen("mediainfo --Output=JSON \""+self.address+'"')
        #res_json=a.read()
        res_json=a.buffer.read().decode('utf-8')
        media_json=json.loads(res_json)

        self.mediainfo_json=media_json

        self.getsubtext()
        self.getaudio()

        self.Format            =media_json['media']['track'][0]['Format']
        self.FileSize          =int(media_json['media']['track'][0]['FileSize'].strip())
        self.duration          =float(media_json['media']['track'][0]['Duration'].strip())
        self.Video_Format      =media_json['media']['track'][1]['Format']
        self.Width             =int(media_json['media']['track'][1]['Width'].strip())
        self.Height            =int(media_json['media']['track'][1]['Height'].strip())
        self.BitDepth          =int(media_json['media']['track'][1]['BitDepth'].strip())
        self.Audio_Format      =media_json['media']['track'][2]['Format']
        if 'Scan type' in media_json['media']['track'][1]:
            self.scan_type=media_json['media']['track'][1]['Scan type']
        else:
            self.scan_type=None
        self.Channels          =int(media_json['media']['track'][2]['Channels'].strip())
        self.standard_sel=None
        if not self.scan_type==None and ( 'inter' in self.scan_type.lower() or '隔行' in self.scan_type):
            if int(self.Height)>2160:
                self.standard_sel='8K'
            elif int(self.Height)>1080:
                self.standard_sel='2160i'
            elif int(self.Height)>720:
                self.standard_sel='1080i'
            elif int(self.Height)>480:
                self.standard_sel='720i'
            else:
                self.standard_sel='480i'
        else:
            if int(self.Height)>2160:
                self.standard_sel='8K'
            elif int(self.Height)>1080:
                self.standard_sel='2160p'
            elif int(self.Height)>720:
                self.standard_sel='1080p'
            elif int(self.Height)>480:
                self.standard_sel='720p'
            else:
                self.standard_sel='480p'

        text=self.mediainfo.lower()
        if ('webrip'in text) or ('web-rip' in text):
            self.type='WEBRip'
        if ('webdl'in text) or ('web-dl' in text):
            self.type='WEB-DL'

        if self.Video_Format == 'AVC':
            self.Video_Format = 'H264'
        if self.Video_Format == 'H264' and 'x264' in text:
            self.Video_Format = 'x264'

        if self.Video_Format == 'HEVC':
            self.Video_Format = 'H265'
        if self.Video_Format == 'H265' and 'x265' in text:
            self.Video_Format = 'x265'
        self.getmediainfo_done=1
        return self.mediainfo

    def dealwith_douban_info(self,infolist):
        self.picture=infolist[0].strip()
        for item in infolist:
            if '译\u3000\u3000名'in item:
                self.allName=item[5:].strip().split('/')
                self.small_descr=item[5:].strip()
            if '片\u3000\u3000名'in item:
                self.name=item[5:].strip()
            if '年\u3000\u3000代'in item:
                self.year=item[5:].strip()
                logger.info('根据豆瓣信息分析，年份为'+self.year)
            if '产\u3000\u3000地'in item:
                self.country=item[5:].strip()
                logger.info('根据豆瓣信息分析，地区为'+self.country)
            if '类\u3000\u3000别'in item:
                self.genre=item[5:].strip()
                logger.info('根据豆瓣信息分析，类别为'+self.genre)
            if '语\u3000\u3000言'in item:
                self.language=item[5:].strip()
                logger.info('根据豆瓣信息分析，语言为'+self.language)
            if '上映日期'in item:
                self.release=item[5:].strip()
                logger.info('根据豆瓣信息分析，上映日期为'+self.release)
            if '首\u3000\u3000播'in item:
                self.firstRelease=item[5:].strip()
                logger.info('根据豆瓣信息分析，首播为'+self.firstRelease)
            if '集\u3000\u3000数'in item:
                self.num=int(item[5:].strip())
                if self.pathinfo.max>=self.num:
                    self.complete=1
                logger.info('根据豆瓣信息分析，总集数为'+str(self.num))
            if self.imdburl=='' and 'imdb'in item and '链接'in item and not((item[7:].strip()).endswith('//')):
                self.imdburl=item[7:].strip()
            if '片\u3000\u3000长'in item:
                self.runtime=item[5:].strip()
    
    def get_douban(self):
        if 'doubancookie' in self.basic and self.basic['doubancookie']!=None:
            res_douban=getdoubaninfo(url=self.doubanurl,cookie=self.basic['doubancookie'],ret_val=True)
        else:
            res_douban=getdoubaninfo(url=self.doubanurl,ret_val=True)
        douban_dict=res_douban.parse()
        self.douban_dict=douban_dict
        self.douban_info=res_douban.info()

        #if (douban_dict['names']['chinesename']):
        #    self.chinesename=douban_dict['names']['chinesename']
        if (douban_dict['names']['akaTitles']):
            self.allName='/'.join(douban_dict['names']['akaTitles'])
        if (douban_dict['year']):
            try:
                self.year=int(douban_dict['year'])
            except:
                logger.warning("douban_dict['year']转换数字出错,其内容为: "+str(douban_dict['year']))
                self.year=2022
        if (douban_dict['countries'] and len(douban_dict['countries']) > 0) :
            self.country=" / ".join(douban_dict['countries'])
        if (douban_dict['genres'] and len(douban_dict['genres']) > 0):
            self.genre=" / ".join(douban_dict['genres'])
        if (douban_dict['languages'] and len(douban_dict['languages']) > 0) :
            if self.language.strip()=='':
                self.language=" / ".join(douban_dict['languages'])
                logger.info('根据豆瓣信息分析，语言为'+self.language)
        if (douban_dict['pubdates'] and len(douban_dict['pubdates']) > 0) :
            self.release=" / ".join(douban_dict['pubdates'])
        if (douban_dict['imdb'].strip()!='') :
            self.imdburl=douban_dict['imdb']
            self.imdburl='https://www.imdb.com/title/'+self.imdburl+'/'
            self.pathinfo.imdb_url=self.imdburl
            logger.info('根据豆瓣信息分析，imdb链接为'+self.imdburl)
        if (douban_dict['episodes']) :
            self.media_type='TV_series'
            self.num=int(douban_dict['episodes'])
            if self.pathinfo.max>=self.num:
                self.complete=1
            logger.info('根据豆瓣信息分析，总集数为'+str(self.num))
        
        self.getptgen_done=1

    def getptgen_douban_info(self):
        if self.getptgen_done==1:
            return
        url='https://api.iyuu.cn/App.Movie.Ptgen?url='+self.doubanurl
        user_agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/98.0.4758.102 Safari/537.36'
        headers = {
                'user-agent': user_agent,
                'referer': url,
            }
        logger.info('正在获取豆瓣信息')
        try:
            r = requests.get(url,headers=headers,timeout=20)
        except Exception as r:
            logger.error('获取豆瓣信息失败，原因: %s' %(r))
            return 

        logger.info('获取豆瓣信息完毕，正在处理信息，请稍等...')
        
        try:
            info_json=r.json()
            logger.trace(info_json)
        except Exception as r:
            logger.warning('获取豆瓣信息转换json格式失败，原因: %s' %(r))
            return
        
        if not r.ok:
            logger.trace(r.content)
            logger.warning(
                f"获取豆瓣信息失败: HTTP {r.status_code}, reason: {r.reason} ")
            return 

        if 'data' not in info_json or 'format' not in info_json['data']:
            logger.warning(f"豆瓣信息获取失败")
            return 
        
        info=info_json['data']['format']
        info=info[0:info.find('<a')]
        self.douban_info=info
        imgurl=re.findall('img[0-9]\.doubanio\.com',self.douban_info)
        if len(imgurl)>0:
            self.douban_info=self.douban_info.replace(imgurl[0],'img9.doubanio.com')
        infolist=info.split('◎')
        self.dealwith_douban_info(infolist)
        self.getptgen_done=1
        


    def getdoubaninfo(self):
        url='https://movieinfo.leaguehd.com/doubanAjax.php?url='+self.doubanurl
        user_agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/98.0.4758.102 Safari/537.36'
        headers = {
                'user-agent': user_agent,
                'referer': url,
            }
        logger.info('正在获取豆瓣信息，请稍等...')
        try:
            r = requests.get(url,headers=headers,timeout=20)
        except Exception as r:
            logger.warning('通过柠檬获取豆瓣信息失败，原因: %s' %(r))
            return

        logger.info('获取豆瓣信息完毕，正在处理信息，请稍等...')
        try:
            data=r.json()
        except Exception as r:
            logger.warning('通过柠檬获取豆瓣信息转换json格式失败，原因: %s' %(r))
            return

        douban_info=''
        if (data['pic']):
            imgurl=re.findall('img[0-9]\.doubanio\.com',data['pic'])
            douban_info = douban_info+"[img]" + data['pic'].replace(imgurl[0],'img9.doubanio.com') + "[/img]\n"
            #self.picture=data['pic']
        if (data['allName']):
            douban_info = douban_info+ "\n◎译\u3000\u3000名　" + '/'.join(data['allName']);
            self.name=data['allName'][0]
            self.allName='/'.join(data['allName'])
        if (data['name']) :
            douban_info += "\n◎片\u3000\u3000名　" + data['name']
            self.name=data['name']
        if (data['year']):
            douban_info += "\n◎年\u3000\u3000代　" + data['year']
            self.year=data['year']
            logger.info('根据豆瓣信息分析，年份为'+self.year)
        if (data['country'] and len(data['country']) > 0) :
            douban_info += "\n◎产\u3000\u3000地　" + " / ".join(data['country'])
            self.country=" / ".join(data['country'])
            logger.info('根据豆瓣信息分析，地区为'+self.country)
        if (data['genre'] and len(data['genre']) > 0):
            douban_info += "\n◎类\u3000\u3000别　" + " / ".join(data['genre'])
            self.genre=" / ".join(data['genre'])
            logger.info('根据豆瓣信息分析，类别为'+self.genre)
        if (data['language'] and len(data['language']) > 0) :
            douban_info += "\n◎语\u3000\u3000言　" + " / ".join(data['language'])
            self.language=" / ".join(data['language'])
            logger.info('根据豆瓣信息分析，语言为'+self.language)
        if (data['release'] and len(data['release']) > 0) :
            douban_info += "\n◎上映日期　" + " / ".join(data['release'])
            self.release=" / ".join(data['release'])
            logger.info('根据豆瓣信息分析，上映日期为'+self.release)
        if (data['firstRelease'] and len(data['firstRelease']) > 0) :
            douban_info += "\n◎首\u3000\u3000播　" + " / ".join(data['firstRelease'])
            self.firstRelease=" / ".join(data['firstRelease'])
            logger.info('根据豆瓣信息分析，首播为'+self.firstRelease)
        if (data['num']) :
            douban_info += "\n◎集\u3000\u3000数　" + data['num']
            self.media_type='TV_series'
            self.num=int(data['num'])
            if self.pathinfo.max>=self.num:
                self.complete=1
            logger.info('根据豆瓣信息分析，总集数为'+str(self.num))
        if (data['imdbRating']) :
            douban_info += "\n◎IMDb评分  " + data['imdbRating'] + "/10 from " + data['imdbVotes'] + " users"
        if (data['imdbUrl']) :
            douban_info += "\n◎IMDb链接  " + data['imdbUrl']
            self.imdburl=data['imdbUrl']
            pathinfo.imdb_url=data['imdbUrl']
            logger.info('根据豆瓣信息分析，imdb链接为'+data['imdbUrl'])
        if (data['rating']) :
            douban_info += "\n◎豆瓣评分　" + data['rating'] + "/10 from " + data['votes'] + " users";
        if (data['url']) :
            douban_info += "\n◎豆瓣链接　" + data['url']
        if (data['runtime'] and len(data['runtime']) > 0) :
            douban_info += "\n◎片　　长　" + " / ".join(data['runtime'])

        if (data['director'] and len(data['director']) > 0) :
            for i in range (len(data['director'])):
                if i==0:
                    douban_info += "\n◎导　　演　" + (data['director'][i]['name'])
                else:
                    douban_info += "\n　　　　　  " + (data['director'][i]['name'])

        if (data['writer'] and len(data['writer']) > 0) :
            for i in range (len(data['writer'])):
                if i==0:
                    douban_info += "\n◎编　　剧　" + (data['writer'][i]['name'])
                else:
                    douban_info += "\n　　　　　  " + (data['writer'][i]['name'])

        if (data['cast'] and len(data['cast']) > 0) :
            for i in range (len(data['cast'])):
                if i==0:
                    douban_info += "\n◎主　　演　" + (data['cast'][i]['name'])
                else:
                    douban_info += "\n　　　　　  " + (data['cast'][i]['name'])

        if (data['tags'] and len(data['tags']) > 0) :
            douban_info += "\n\n\n◎标　　签　" + " | ".join(data['tags'])
        if (data['plot']) :
            douban_info=douban_info[0:douban_info.find('<a')]
            plotstr=data['plot']
            plotstr=plotstr[0:plotstr.find('<a')]
            douban_info += "\n\n◎简　　介　" + "\n\n " +(plotstr)

        if (data['awards'] and len(data['awards']) > 0) :
            awardstr=''
            for item in data['awards']:
                awardstr=awardstr+"\n\n　　" + item['title'];
                for itemc in item['content']:
                    awardstr=awardstr+"\n　　" + itemc
            douban_info += "\n\n◎获奖情况　" + awardstr

        douban_info =douban_info+ "\n\n"

        self.douban_info=douban_info
        self.getptgen_done=1

    def mktorrent(self,tracker='https://announce.leaguehd.com/announce.php'):
        #if self.mktorrent_done==1:
        #    return
        torrentpath=os.path.join(self.screenshotaddress,str(self.episode)+'.torrent')
        self.torrentpath=torrentpath
        mktorrent(self.topath,torrentpath,tracker=tracker)
        self.mktorrent_done=1

    def gettorrent(self,tracker='https://announce.leaguehd.com/announce.php'):
        dirpath=os.path.dirname(self.topath)
        filelist=[]
        if not os.path.exists(self.topath):
            os.makedirs(self.topath)
        else:
            ls = os.listdir(self.topath)
            for i in ls:
                c_path=os.path.join(self.topath, i)
                if i.startswith('.'):
                    os.remove(c_path)
                    continue
                if (os.path.isdir(c_path)):
                    if not os.path.exists(   os.path.join(dirpath,i)    ):
                        newpath=move(c_path,dirpath)
                        filelist.append(newpath)
                    else:
                        logger.warning('由于文件'+c_path+'在里外文件夹均已存在,已改名为_temp')
                        os.rename(c_path,c_path+'_temp')
                        newpath=move(c_path+'_temp',dirpath)
                        filelist.append(newpath)
                else:
                    if not os.path.exists(   os.path.join(dirpath,i)    ):
                        newpath=move(c_path,dirpath)
                        filelist.append(newpath)
                    else:
                        logger.warning('由于文件'+c_path+'在里外文件夹均已存在,已改名为_temp')
                        stem, suffix = os.path.splitext(c_path)
                        os.rename(c_path,stem+'_temp'+suffix)
                        newpath=move(stem+'_temp'+suffix,dirpath)
                        filelist.append(newpath)

        logger.info('检测到路径制种，将先删除掉路径里面所有种子文件(torrent后缀)以及隐藏文件（.开头的文件）...')
        deletetorrent(self.topath) 
        if os.path.isdir(self.mediapath):
            ls = os.listdir(self.mediapath)
            for i in ls:
                c_path=os.path.join(self.mediapath, i)
                if (os.path.isdir(c_path)) or (i.startswith('.')) or (not(  os.path.splitext(i)[1].lower()== ('.mp4') or os.path.splitext(i)[1].lower()== ('.mkv')  or os.path.splitext(i)[1].lower()== ('.avi') or os.path.splitext(i)[1].lower()== ('.ts')    )):
                    continue
                move (c_path,self.topath)
        else:
            move (self.mediapath,self.topath)

        self.mktorrent(tracker)

        for item in filelist:
            if os.path.exists(item):
                newpath=move(item,self.topath)
                if '_temp' in newpath and os.path.exists(newpath):
                    os.rename(newpath,newpath.replace('_temp',''))



    def getfullinfo(self,tracker='https://announce.leaguehd.com/announce.php'):
        if self.getinfo_done==1:
            return
        
        trytime=0
        while not self.getptgen_done==1:
            logger.info('正在获取豆瓣信息...')
            trytime += 1
            if trytime>10:
                logger.error('获取豆瓣信息失败')
                raise Exception('获取豆瓣信息失败')
            self.get_douban()
            '''
            if self.getptgen_done<1:
                self.getptgen_douban_info()
            if self.getptgen_done<1:
                self.getdoubaninfo()
            '''
            if self.getptgen_done<1:
                time.sleep(3)
        


        self.getmediainfo()

        if self.pathinfo.year!='':
            self.year=self.pathinfo.year
        if self.pathinfo.video_type!='':
            self.type=self.pathinfo.video_type
        if self.pathinfo.video_format!='':
            self.Video_Format=self.pathinfo.video_format
        if self.pathinfo.audio_format!='':
            self.Audio_Format=self.pathinfo.audio_format
        if self.pathinfo.txt_info!='':
            self.sublan='['+self.pathinfo.txt_info+']'
        if self.pathinfo.audio_info!='':
            self.language=self.pathinfo.audio_info


        self.uploadname=self.englishname+' '+str(self.year)
        self.small_descr=self.douban_dict['names']['chinesename'].strip()

        
        medianame=self.uploadname
        if self.pathinfo.type=='anime' or self.pathinfo.type=='tv':
            self.uploadname=self.uploadname+' '+self.season
            #if int(self.seasonnum)>1:
            #self.small_descr=self.small_descr+' | '+self.season_ch.strip()
            medianame=self.uploadname
            if not self.isdir:
                self.uploadname=self.uploadname+'E'+self.episodename
                self.small_descr=self.small_descr+' | 第E'+self.episodename+'集'
            elif self.complete==1:
                self.uploadname=self.uploadname
                self.small_descr=self.small_descr+' | 全 '+str(self.pathinfo.max)+' 集'
            else:
                self.uploadname=self.uploadname+'E'+str(self.pathinfo.min).zfill(2)+'-E'+str(self.pathinfo.max).zfill(2)
                self.small_descr=self.small_descr+' | 第E'+str(self.pathinfo.min).zfill(2)+'-E'+str(self.pathinfo.max).zfill(2)+'集'


        if self.douban_dict['directors']:
            self.small_descr=self.small_descr+' | 导演: '+self.douban_dict['directors'][0]['name'].strip()
            for i in range(min(len(self.douban_dict['directors'])-1,1)):
                self.small_descr=self.small_descr+' / '+self.douban_dict['directors'][i+1]['name'].strip()

        if self.douban_dict['actors']:
            self.small_descr=self.small_descr+' | 主演: '+self.douban_dict['actors'][0]['name'].strip()
            for i in range(min(len(self.douban_dict['actors'])-1,4)):
                self.small_descr=self.small_descr+' / '+self.douban_dict['actors'][i+1]['name'].strip()

        medianame = medianame+' '+self.standard_sel+' '+self.type+' '+self.Video_Format+' '+self.Audio_Format+'-'+self.sub
        while '  'in medianame:
            medianame=medianame.replace('  ',' ')
        medianame=medianame.replace(' ','.')
        

        

        if 'new_folder' in self.basic and self.basic['new_folder']==1:
            if self.pathinfo.zeroday_name!='':
                self.topath=os.path.join(self.pathinfo.path,self.pathinfo.zeroday_name)
            else:
                self.topath=os.path.join(self.pathinfo.path,medianame)
                self.pathinfo.infodict['zeroday_name']=medianame
        elif 'new_folder' in self.basic and self.basic['new_folder']==2:
            if self.pathinfo.zeroday_name!='':
                self.topath=os.path.join(self.pathinfo.path,self.pathinfo.zeroday_name)
            else:
                tempchinesename=self.chinesename.strip()
                while '  'in tempchinesename:
                    tempchinesename=tempchinesename.replace('  ',' ')
                tempchinesename=tempchinesename.replace(' ','.')
                self.topath=os.path.join(self.pathinfo.path,tempchinesename+'.'+medianame)
                self.pathinfo.infodict['zeroday_name']=tempchinesename+'.'+medianame
                del(tempchinesename)
        else:
            self.topath=self.mediapath

            

        self.uploadname_ssd=self.uploadname+' '+self.type+' '+self.standard_sel+' '+self.Video_Format+' '+self.Audio_Format+(self.audio_num>1)*('.'+str(self.audio_num)+'Audio') +'-'+self.sub
        self.uploadname    =self.uploadname+' '+self.standard_sel+' '+self.type+' '+self.Video_Format+' '+self.Audio_Format+(self.audio_num>1)*(' '+str(self.audio_num)+'Audio') +'-'+self.sub
        
        try:
            if self.language!='':
                self.small_descr=self.small_descr+' ['+self.language+']'
        except:
            logger.warning('未找到资源语言信息，默认为日语')
            self.small_descr=self.small_descr+' [日语]'

        if not self.sublan=='':
            self.small_descr=self.small_descr+' '+self.sublan
        else:
            self.small_descr=self.small_descr+' [无字幕]'
        self.small_descr=self.small_descr.replace('（','(').replace('）',')')

        if self.pathinfo.small_descr!='':
            self.small_descr=self.pathinfo.small_descr
        #logger.debug('副标题为'+self.small_descr)
        self.getimgurl()
        if self.pathinfo.screenshot!='':
            self.screenshoturl=self.pathinfo.screenshot+'\n'+self.screenshoturl
        self.content=self.douban_info+"\n[quote=Mediainfo]\n"+self.mediainfo+"[/quote]\n"+self.screenshoturl

        if self.pathinfo.contenthead!='':
            self.content= self.pathinfo.contenthead+self.content
        
        if self.pathinfo.contenttail!='':
            self.content= self.content+self.pathinfo.contenttail

        
        if 'new_folder' in self.basic and self.basic['new_folder']>=1:
            self.gettorrent(tracker)
        else:
            self.mktorrent(tracker)
        self.getinfo_done=1

    def print(self):
        self.getfullinfo()
        attr=['uploadname','small_descr','content']
        for item in attr:
            exec('print("'+item+':"  ,self.'+item+'  )')





from setuptools import setup, find_packages
import io

setup(
    name = "upload_machine",     
    version = "0.0.18", 
    keywords = ["pip", "autoupload","auto","upload","PT","private tracker"],            
    description = "Upload local resources to PT trackers automatically.",    
    long_description=io.open("README.md", "r", encoding='utf-8').read(),
    long_description_content_type="text/markdown",
    license = "MIT Licence",    

    entry_points = {
        'console_scripts': [
            'upload_machine=upload_machine.main:main',
            'um=upload_machine.main:main',
        ],
    },

    url = "https://github.com/dongshuyan/Upload_Machine", 
    author = "sauterne",            
    author_email = "ssauterne@qq.com",

    packages = find_packages(),
    include_package_data = True,
    exclude_package_data = {'': ['__pycache__']},

    platforms = "any",
    python_requires = '>=3.9.0',
    install_requires = ["loguru","pathlib","typing","lxml","pyyaml","requests","bs4","datetime","qbittorrent-api","function_controler","doubaninfo","cloudscraper"]
)


import os
from loguru import logger
from urllib.parse import quote
from urllib.parse import unquote
import re
import requests
from upload_machine.utils.para_ctrl.readyaml import write_yaml
from shutil import move

def findnum(name):
    num=re.findall(r" ([0-9]{1,2}) ",name)
    if len(num)!=0:
        sty=" "+num[0]+" "
        stz=" XX "
        stx=" "+num[0]+" "
        return num[0],sty,stz,stx
    num=re.findall(r"第([0-9]{1,2})話",name)
    if len(num)!=0:
        sty='第'+num[0]+'話'
        stz="第XX話"
        stx='第'+num[0]+'話'
        return num[0],sty,stz,stx
    num=re.findall(r"第([0-9]{1,2})话",name)
    if len(num)!=0:
        sty='第'+num[0]+'话'
        stz="第XX话"
        stx='第'+num[0]+'话'
        return num[0],sty,stz,stx
    num=re.findall(r"E([0-9]{1,2})",name)
    if len(num)!=0:
        sty='第'+num[0]+'话'
        stz="第XX话"
        stx='第'+num[0]+'话'
        return num[0],sty,stz,stx
    num=re.findall(r"_([0-9]{1,2})_",name)
    if len(num)!=0:
        sty='_'+num[0]+'_'
        stz="_XX_"
        stx='_'+num[0]+'_'
        return num[0],sty,stz,stx
    num=re.findall(r"\[([0-9]{1,2})\D",name)
    if len(num)!=0:
        sty="\["+num[0]+"\]"
        stz="[XX]"
        stx="["+num[0]+"]"
        return num[0],sty,stz,stx
    num=re.findall(r"\D([0-9]{1,2})\]",name)
    if len(num)!=0:
        sty="\["+num[0]+"\]"
        stz="[XX]"
        stx="["+num[0]+"]"
        return num[0],sty,stz,stx
    num=re.findall(r"-([0-9]{1,2})\D",name)
    if len(num)!=0:
        sty='-'+num[0]
        stz="-XX"
        stx="-"+num[0]
        return num[0],sty,stz,stx
    num=re.findall(r"\D([0-9]{1,2})\D",name)
    if len(num)!=0:
        sty=num[0]
        stz="XX"
        stx=num[0]
        return num[0],sty,stz,stx  
    
    num=re.findall(r"([0-9]{1,2})",name)
    if len(num)!=0:
        sty=num[0]
        stz="XX"
        stx=num[0]
        return num[0],sty,stz,stx     
    return '-1','-1','XX','-1'

def finddoubanurl(name):
    logger.info('正在寻找 '+name+' 的豆瓣链接，请稍等...')
    url_encode_name = quote(name.replace(' ',''))
    url='https://www.douban.com/search?q='+url_encode_name
    user_agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/98.0.4758.102 Safari/537.36'
    headers = {
            'user-agent': user_agent,
            'referer': url,
            }
    r=requests.models.Response()
    try:
        r = requests.get(url,headers=headers,timeout=30)
    except:
        logger.warning('寻找豆瓣链接失败')
        
    for item in re.findall('/?url=(.*)%2F',r.text):
        item=unquote(item)
        if ('movie.douban.com'in item) and ('/subject/' in item or '/movie/' in item) :
            logger.info('已找到 '+name+' 的豆瓣链接为:\n'+item+'\n')
            res=input('请确认豆瓣链接正确性,如果正确回复y,否则回复正确的豆瓣链接:\n')
            if res.strip().lower()=='y':
                logger.info('已确认 '+name+' 的豆瓣链接为:'+item+'\n')
                return item
            else :
                logger.info('已确认 '+name+' 的豆瓣链接为:'+res+'\n')
                return res
    return ''

def findbgmurl(name):
    logger.info('正在寻找 '+name+' 的Bangumi链接，请稍等...')
    url_encode_name = quote(name)
    url='https://bgm.tv/subject_search/'+url_encode_name+'?cat=2'
    user_agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/98.0.4758.102 Safari/537.36'
    headers = {
            'user-agent': user_agent,
            'referer': url,
            }
    r = requests.get(url,headers=headers,timeout=30)
    ans=re.findall('href="/subject/(\d*)',r.text)
    if len(ans)>0:
        logger.info('已找到 '+name+' 的Bangumi链接为:\n'+'https://bgm.tv/subject/'+ans[0]+'\n')
        res=input('请确认Bangumi链接正确性,如果正确回复y,否则回复正确的Bangumi链接,若不需要则直接输入回车:\n')
        if res.strip().lower()=='y':
            logger.info('已确认 '+name+' 的Bangumi链接为:'+'https://bgm.tv/subject/'+ans[0]+'\n')
            return 'https://bgm.tv/subject/'+ans[0]
        else :
            logger.info('已确认 '+name+' 的Bangumi链接为:'+res+'\n')
            return res
    return ''

def findeps(pathlist):
    eps=[]
    for path in pathlist:
        if not os.path.exists(path):
            continue
        ls = os.listdir(path)
        ls.sort()
        for i in ls:
            c_path = os.path.join(path, i)
            if (os.path.isdir(c_path)) or (i.startswith('.')) or (not(  os.path.splitext(i)[1].lower()== ('.mp4') or os.path.splitext(i)[1].lower()== ('.mkv')  or os.path.splitext(i)[1].lower()== ('.avi') or os.path.splitext(i)[1].lower()== ('.ts')    )):
                continue
            epnum=int(findnum(i)[0])
            if epnum==-1:
                logger.error(c_path+' 这个文件，文件名为:'+i+',没找到集数信息，请用户检查，如果确定是程序没有抽取成功请联系作者')
                raise ValueError (c_path+' 这个文件，文件名为:'+i+',没找到集数信息，请用户检查，如果确定是程序没有抽取成功请联系作者')
            if not epnum in eps:
                eps.append(epnum)
    eps.sort()
    return eps





class pathinfo(object):
    def __init__(self,pathid,infodict,sites,basic):
        self.pathid=pathid
        self.sites=[]
        self.exclusive=[]
        self.infodict=infodict
        self.max=1
        self.min=1
        #必须有的属性
        attr_must=['path','chinesename','englishname','sub']
        for item in attr_must:
            if not item in infodict or infodict[item]==None:
                logger.error('未识别'+pathid+' 中的'+item+'信息')
                raise ValueError ('未识别'+pathid+' 中的'+item+'信息')
            else:
                exec('self.'+item+'=infodict[item]')
                exec('self.exist_'+item+'=True')

        

        #可有可无的属性,后面写入配置文件
        attr_disp=['type','collection','complete','enable','doubanurl']
        for item in attr_disp:
            if not item in infodict or infodict[item]==None:
                exec('self.'+item+'=""')
                exec('infodict[item]=None')
                exec('self.exist_'+item+'=False')
            else:
                exec('self.'+item+'=infodict[item]')
                exec('self.exist_'+item+'=True')

        #可有可无的属性,后面不写入配置文件
        attr_disp=['video_type','video_format','audio_format','year','zeroday_name','exinfo','seasonnum','imdb_url','bgm_url','anidb_url','transfer','txt_info','audio_info','from_url','contenttail','contenthead','screenshot','small_descr']
        for item in attr_disp:
            if not item in infodict or infodict[item]==None:
                exec('self.'+item+'=""')
                exec('self.exist_'+item+'=False')
            else:
                exec('self.'+item+'=infodict[item]')
                exec('self.exist_'+item+'=True')
        
        
        pathstr=os.path.basename(self.path)
        if (self.exist_chinesename and self.exist_englishname and self.exist_sub):
            if self.exinfo!='':
                self.exinfo='['+self.exinfo+']'
            if self.seasonnum=='':
                self.seasonnum=1
                self.season='S01'
            else:
                try:
                    self.seasonnum=int(self.seasonnum)
                    self.season='S'+str(self.seasonnum).zfill(2)
                except:
                    logger.error('季度输入错误')
                    raise ValueError (pathid+'季度输入错误')

        elif   (len(pathstr.split('-'))==4)  :

            path1=os.path.basename(self.path)
            if self.seasonnum!='':
                try:
                    self.seasonnum=int(self.seasonnum)
                    self.season='S'+str(self.seasonnum).zfill(2)
                except:
                    logger.error('季度输入错误')
                    raise ValueError (pathid+'季度输入错误')
            if self.sub=='':
                self.sub               = path1.split('-')[-1].strip()
                infodict['sub']=self.sub
            if self.englishname=='': 
                self.englishname       = path1.split('-')[-2].strip()
                seasons=re.findall("S[0-9]{1,2}",self.englishname) 
                if len(seasons)==0 and self.seasonnum=='':
                    self.season='S01'
                    self.seasonnum=1

                else:
                    if self.seasonnum=='':
                        self.seasonnum=int(seasons[0][1:])
                        self.season='S'+str(self.seasonnum).zfill(2)
                        infodict['seasonnum']=self.seasonnum
                    self.englishname=self.englishname.replace(self.season,'').strip()
                infodict['englishname']=self.englishname
            if self.chinesename=='':
                self.chinesename       =path1.split('-')[-3].strip()
                if len(re.findall("第.*季",self.chinesename))>0:
                    self.chinesename=self.chinesename.replace(re.findall("第.*季",self.chinesename)[0],'')
                    self.chinesename=self.chinesename.strip()
                infodict['chinesename']=self.chinesename
            if self.exinfo=='':
                self.exinfo            =re.findall('\[.*\]',pathstr.split('-')[0].strip())
                if len(self.exinfo)>0:
                    self.exinfo=self.exinfo[0]
                    infodict['exinfo']=self.exinfo
                else:
                    self.exinfo=''
            else:
                self.exinfo='['+self.exinfo+']'

            

        else:
            logger.error(pathid+' 信息异常，中文名，英文名，制作小组名有缺失，请补齐后重发')
            raise ValueError (pathid+' 信息异常，中文名，英文名，制作小组名有缺失，请补齐后重发')
        
        pathitem=self.chinesename

        if not self.exist_type:
            logger.warning('未识别路径'+infodict['path']+'的type（资源类型）信息')
            res=100
            while not(res>0 and res<4):
                res=input('请输入资源类型对应的数字:\n1.动漫 2.电视剧 3.电影\n')
                try:
                    res=int(res)
                except:
                    res=100
                if not(res>0 and res<4):
                    logger.warning('输入有误，请重新输入')
            if res==1:
                self.type='anime'
                infodict['type']='anime'
            elif res==2:
                self.type='tv'
                infodict['type']='tv'
            elif res==3:
                self.type='movie'
                infodict['type']='movie'
            else:
                logger.error(pathid+' 中type(资源类型)输入错误')
                raise ValueError (pathid+' 中type(资源类型)输入错误')


        self.downloadpath=''
        if not 'downloadpath' in infodict or infodict['downloadpath']==None:
            if (self.collection or 'movie' in self.type) == 1 and ('new_folder' in basic and basic['new_folder']==0):
                self.downloadpath=os.path.dirname( self.path)
            else:
                self.downloadpath=self.path
        else:
            self.downloadpath=infodict['downloadpath']

        self.category=None
        if not 'category' in infodict or infodict['category']==None:
            self.category=None
        else:
            self.category=infodict['category']

        if self.complete=='':
            self.complete=0
        else:
            self.complete=int(self.complete)

        
        if ('anime' in self.type or 'tv' in self.type) and not self.exist_collection:
            res=100
            while not(res==0 or res==1):
                res=input('未识别路径'+pathid+'的collection（资源是否按合集发布）信息,请重新输入选项对应的数字:\n0:发布单集,1:发布合集\n')
                try:
                    res=int(res)
                except:
                    res=100
                if not(res==0 or res==1):
                    logger.warning('输入有误，请重新输入')
            if res==0:
                self.collection =0
                infodict['collection']=0
            elif res==1:
                self.collection =1
                infodict['collection']=1
            else:
                logger.error('未识别路径'+pathid+'的collection（资源是否按合集发布）信息')
                raise ValueError ('未识别路径'+pathid+'的collection（资源是否按合集发布）信息')


        if self.exist_enable:
            try:
                self.enable =int(self.enable)
            except:
                logger.warning('未识别路径'+pathid+'的enable（资源是否发布）信息,已设置为0（不发）')
                self.enable =0
                infodict['enable']=0
            if not (self.enable==0 or self.enable==1):
                logger.warning('未识别路径'+pathid+'的enable（资源是否发布）信息,已设置为0（不发）')
                self.enable =0
                infodict['enable']=0
        else:
            logger.warning('未识别路径'+pathid+'的enable（资源是否发布）信息,已设置为0（不发）')
            self.enable =0
            infodict['enable']=0

        if self.exist_collection:
            try:
                self.collection =int(self.collection)
            except:
                logger.warning('未识别路径'+pathid+'的collection（资源是否以合集发布）信息,已设置为0（单集发布）')
                self.collection =0
                infodict['collection']=0
            if not (self.collection==0 or self.collection==1):
                logger.warning('未识别路径'+pathid+'的collection（资源是否以合集发布）信息,已设置为0（单集发布）')
                self.collection =0
                infodict['collection']=0
        else:
            logger.warning('未识别路径'+pathid+'的collection（资源是否以合集发布）信息,已设置为0（单集发布）')
            self.collection =0
            infodict['collection']=0

        if self.exist_transfer:
            try:
                self.transfer =int(self.transfer)
            except:
                logger.warning('未识别路径'+pathid+'的transfer(资源是否为转载)信息,已设置为1（默认资源为转载）')
                self.transfer =1
                infodict['transfer']=1
            if not (self.transfer==0 or self.transfer==1):
                logger.warning('未识别路径'+pathid+'的transfer(资源是否为转载)信息,已设置为1（默认资源为转载）')
                self.transfer =1
                infodict['transfer']=1
        else:
            logger.warning('未识别路径'+pathid+'的transfer(资源是否为转载)信息,已设置为1（默认资源为转载）')
            self.transfer =1
            infodict['transfer']=1

        if self.transfer==1 and not self.exist_from_url:
            logger.warning('未识别路径'+pathid+'from_url(转载资源原链接)信息')
            self.from_url = 'https://mikanani.me/'
            infodict['from_url']='https://mikanani.me/'

        if self.exist_complete:
            try:
                self.complete =int(self.complete)
            except:
                logger.warning('未识别路径'+pathid+'的complete(资源是否为完结)信息,已设置为0（未完结）')
                self.complete =0
                infodict['complete']=0
            if not (self.complete==0 or self.complete==1):
                logger.warning('未识别路径'+pathid+'的complete(资源是否为完结)信息,已设置为0（未完结）')
                self.complete =0
                infodict['complete']=0
        else:
            logger.warning('未识别路径'+pathid+'的complete(资源是否为完结)信息,已设置为0（未完结）')
            self.complete =0
            infodict['complete']=0
            


        
        for siteitem in sites:
            if sites[siteitem].enable==0:
                continue
            self.sites.append(siteitem)
            if (not siteitem in infodict) or (infodict[siteitem]==None):
                logger.warning('未找到路径'+pathitem+'在'+siteitem+'的站点信息,已设置为不发')
                exec('self.'+siteitem+'_done=[]')
                exec('self.'+siteitem+'_max_done=10000')
                exec('self.'+siteitem+'_min_done=-1')
                infodict[siteitem]=None
            else:
                exec('self.'+siteitem+'_done='+'str(infodict[siteitem]).split(",")')
                exec('self.'+siteitem+'_max_done=-1')
                exec('self.'+siteitem+'_min_done=10000')
                for i in range(eval('len(self.'+siteitem+'_done)')):
                    #self.done[i]= int(self.done[i])
                    exec('self.'+siteitem+'_done[i]='+'int(self.'+siteitem+'_done[i])')
                    if eval('self.'+siteitem+'_done[i]>self.'+siteitem+'_max_done'):
                        exec('self.'+siteitem+'_max_done=self.'+siteitem+'_done[i]')
                    if eval('self.'+siteitem+'_done[i]<self.'+siteitem+'_min_done'):
                        exec('self.'+siteitem+'_min_done=self.'+siteitem+'_done[i]')
                exec('self.'+siteitem+'_done.sort()')
        
        if 'exclusive' in infodict and infodict['exclusive']!=None:
            self.exclusive=infodict['exclusive'].split(",")
            self.exclusive=[i.strip().lower() for i in self.exclusive]

        season_ch=''
        season_ch=season_ch+'第'
        if self.seasonnum==1:
            season_ch=season_ch+'一'
        elif self.seasonnum==2:
            season_ch=season_ch+'二'
        elif self.seasonnum==3:
            season_ch=season_ch+'三'
        elif self.seasonnum==4:
            season_ch=season_ch+'四'
        elif self.seasonnum==5:
            season_ch=season_ch+'五'
        elif self.seasonnum==6:
            season_ch=season_ch+'六'
        elif self.seasonnum==7:
            season_ch=season_ch+'七'
        elif self.seasonnum==8:
            season_ch=season_ch+'八'
        elif self.seasonnum==9:
            season_ch=season_ch+'九'
        elif self.seasonnum==10:
            season_ch=season_ch+'十'
        elif self.seasonnum==11:
            season_ch=season_ch+'十一'
        elif self.seasonnum==12:
            season_ch=season_ch+'十二'
        elif self.seasonnum==13:
            season_ch=season_ch+'十三'
        elif self.seasonnum==14:
            season_ch=season_ch+'十四'
        elif self.seasonnum==15:
            season_ch=season_ch+'十五'
        else:
            season_ch=season_ch+str(self.seasonnum)
        season_ch=season_ch+'季'
        self.season_ch=season_ch
        
        if ('anime' in self.type.lower() or 'tv' in self.type.lower()):
            
            

            if self.zeroday_name=='':
                self.eps=findeps([self.path])
            else:
                self.eps=findeps([self.path,os.path.join(self.path,self.zeroday_name)])
            if (len(self.eps)<1):
                raise Exception('路径'+pathid+' : '+self.infodict['path']+'中没找到视频文件')
            self.min=self.eps[0]
            self.max=self.eps[-1]
            '''
            if (not self.exist_bgm_url) and 'anime' in self.type:
                if self.seasonnum>1:
                    self.bgm_url=findbgmurl(self.chinesename.strip()+' '+self.season_ch.strip())
                else:
                    self.bgm_url=findbgmurl(self.chinesename.strip())
                if self.bgm_url=='':
                    #logger.error('未找到 '+self.chinesename+' 对应的Bangumi地址。')
                    logger.warning('未找到 '+self.chinesename+' 对应的Bangumi地址。暂设置为空，如果需要请手动前往au.yaml设置')
                    infodict['bgm_url']=None
                else:
                    infodict['bgm_url']=self.bgm_url
            '''

        if (not self.exist_doubanurl):
            if ('anime' in self.type or 'tv' in self.type):
                if self.seasonnum>1:
                    self.doubanurl=finddoubanurl(self.chinesename.strip()+' '+self.season_ch.strip())
                else:
                    self.doubanurl=finddoubanurl(self.chinesename.strip())
            else:
                self.doubanurl=finddoubanurl(self.chinesename.strip())
            if self.doubanurl=='':
                logger.warning('未找到 '+self.chinesename+' 对应的豆瓣地址')
                res=input('未找到 '+self.chinesename+' 对应的豆瓣地址。请手动输入正确的豆瓣链接:\n')
                self.doubanurl=res
                infodict['doubanurl']=res
            else:
                infodict['doubanurl']=self.doubanurl




        if ('anime' in self.type or 'tv' in self.type):
            if len(self.eps) == self.max-self.min+1:
                self.lack=False
            else:
                self.lack=True
            if self.lack:
                self.lackeps=[]
                for i in range(self.min,self.max+1):
                    if not i in self.eps:
                        self.lackeps.append(i)
                logger.warning('识别到路径'+pathid+' 中资源存在部分集数缺失,缺失集数:'+str(self.lackeps))
                res=100
                while not(res==0 or res==1):
                    res=input('识别到路径'+pathid+' 中资源存在部分集数缺失,缺失集数为:'+str(self.lackeps)+'。是否仍然继续发布？请回复选项对应的数字\n0:先不发布，退出程序 1:无视警告,仍然发布\n')
                    try:
                        res=int(res)
                    except:
                        res=100
                    if not(res==0 or res==1):
                        logger.warning('输入有误，请重新输入')
                if res==0:
                    logger.error('识别到路径'+pathid+' 中资源存在部分集数缺失,用户退出程序')
                    raise ValueError ('识别到路径'+pathid+' 中资源存在部分集数缺失,用户退出程序')

            

    def print(self):
        attr=['path','type','collection','enable','doubanurl','imdb_url','bgm_url','anidb_url','from_url','transfer','sub','englishname','chinesename']
        print('Path info:')
        for item in attr:
            exec('print("'+item+':"  ,self.'+item+'  )')
        

        series_attr=['season','seasonnum','eps','min','max']
        if (self.type=='anime' or self.type=='tv'):
            for item in series_attr:
                exec('print("'+item+':"  ,self.'+item+'  )')
            for item in self.sites:
                exec('print("'+item+'_done:"  ,self.'+item+'_done  )')
                exec('print("'+item+'_min_done:"  ,self.'+item+'_min_done  )')
                exec('print("'+item+'_max_done:"  ,self.'+item+'_max_done  )')

        if ('anime' in self.type or 'tv' in self.type) :
            print('self.lack:',self.lack)
            if self.lack:
                print('self.lackeps:',self.lackeps)
        print('')


def findpathinfo(yamlinfo,sites):
    paths=yamlinfo['path info']
    pathlist=[]
    for item in paths:
        pathlist.append(pathinfo(item,paths[item],sites,yamlinfo['basic']))
    write_yaml(yamlinfo)
    #a=input('checkpath')
    return pathlist





import os
from loguru import logger
from upload_machine.utils.img_upload.chevereto import chevereto_api_upload_files,chevereto_cookie_upload_files
from upload_machine.utils.img_upload.ptpimg import ptpimg_upload_files
from upload_machine.utils.img_upload.smms import smms_upload_files
from upload_machine.utils.img_upload.fapping_emp import femp_upload_files
from upload_machine.utils.img_upload.imgbox import imgbox_upload_files


def existitem(imgdata,item):
    if item in imgdata and not(imgdata[item]=='') and not imgdata[item]==None:
        return True
    return False

def createimgdict(imgdata):
    imgdict=dict()
    imghostlist=['ptpimg','picgo','smms','pter','emp','femp','imgbox','chd','freeimage']
    for item in imghostlist:
        imgdict[item]=False
    if existitem(imgdata,'ptpimg') and existitem(imgdata['ptpimg'],'apikey'):
        imgdict['ptpimg']=True
    if existitem(imgdata,'picgo') and existitem(imgdata['picgo'],'apikey') and existitem(imgdata['picgo'],'url') :
        imgdict['picgo']=True
    if existitem(imgdata,'smms') and existitem(imgdata['smms'],'apikey'):
        imgdict['smms']=True
    if existitem(imgdata,'pter')  and existitem(imgdata['pter'],'url') and existitem(imgdata['pter'],'cookie') :
        imgdict['pter']=True
    if existitem(imgdata,'emp')  and existitem(imgdata['emp'],'url') and existitem(imgdata['emp'],'cookie') :
        imgdict['emp']=True
    if existitem(imgdata,'chd')  and existitem(imgdata['chd'],'url') and existitem(imgdata['chd'],'cookie') :
        imgdict['chd']=True
    if existitem(imgdata,'freeimage')  and existitem(imgdata['freeimage'],'url') and existitem(imgdata['freeimage'],'cookie') :
        imgdict['freeimage']=True
    imgdict['femp']=True
    imgdict['imgbox']=True
    listnum=1
    seq=[]
    while (existitem(imgdata['seq'],(listnum))):
        seq.append(imgdata['seq'][(listnum)])
        listnum=listnum+1
    return imgdata,imgdict,seq

def img_upload_seq(imgdata,seq:list[str],imglist:list[str],form:str='img'):
    for i in range(len(seq)):
        res=img_upload(imgdata=imgdata,imglist=imglist,host=seq[i],form=form,fail=True)
        if not (res==''):
            return res

def img_upload(imgdata,imglist:list[str],host:str='',form:str='img',fail:bool=False,chevereto_url:str='',chevereto_apikey:str='',chevereto_cookie:str=''):
    imgdata,imgdict,seq=createimgdict(imgdata)
    success=True
    res=''
    if host=='':
    	host=seq[0]
    if 'ptpimg'in host.lower():
        host='ptpimg'
        logger.info('正在尝试使用'+host+'上传图片,请稍等...')
        if imgdict[host]==True:
            res=ptpimg_upload_files(imgpaths=imglist,api_key=imgdata[host]['apikey'],form=form)
        else:
            success=False
            logger.warning('图床'+host+'配置信息缺失')
        if res=='':
            success=False
    elif 'picgo' in host.lower():
        host='picgo'
        logger.info('正在尝试使用'+host+'上传图片,请稍等...')
        if imgdict[host]==True:
            res=chevereto_api_upload_files(imgpaths=imglist,url=imgdata[host]['url'].strip('/'),api_key=imgdata[host]['apikey'],form=form)
        else:
            success=False
            logger.warning('图床'+host+'配置信息缺失')
        if res=='':
            success=False
    elif 'smms' in host.lower():
        host='smms'
        logger.info('正在尝试使用'+host+'上传图片,请稍等...')
        if imgdict[host]==True:
            res=smms_upload_files(imgpaths=imglist,api_key=imgdata[host]['apikey'],form=form)
        else:
            success=False
            logger.warning('图床'+host+'配置信息缺失')
        if res=='':
            success=False
    elif 'pter' in host.lower():
        host='pter'
        logger.info('正在尝试使用'+host+'上传图片,请稍等...')
        if imgdict[host]==True:
            res=chevereto_cookie_upload_files(imgpaths=imglist,url=imgdata[host]['url'].strip('/'), cookie=imgdata[host]['cookie'], form=form)
        else:
            success=False
            logger.warning('图床'+host+'配置信息缺失')
        if res=='':
            success=False
    elif 'femp' in host.lower():
        host='femp'
        logger.info('正在尝试使用'+host+'上传图片,请稍等...')
        if imgdict[host]==True:
            res=femp_upload_files(imgpaths=imglist, form=form)
        else:
            success=False
            logger.warning('图床'+host+'配置信息缺失')
        if res=='':
            success=False
    elif 'emp' in host.lower():
        host='emp'
        logger.info('正在尝试使用'+host+'上传图片,请稍等...')
        if imgdict[host]==True:
            res=chevereto_cookie_upload_files(imgpaths=imglist,url=imgdata[host]['url'].strip('/'), cookie=imgdata[host]['cookie'], form=form)
        else:
            success=False
            logger.warning('图床'+host+'配置信息缺失')
        if res=='':
            success=False
    elif 'imgbox' in host.lower():
        host='imgbox'
        logger.info('正在尝试使用'+host+'上传图片,请稍等...')
        if imgdict[host]==True:
            res=imgbox_upload_files(imgpaths=imglist, form=form)
        else:
            success=False
            logger.warning('图床'+host+'配置信息缺失')
        if res=='':
            success=False
    elif 'chd' in host.lower():
        host='chd'
        logger.info('正在尝试使用'+host+'上传图片,请稍等...')
        if imgdict[host]==True:
            res=chevereto_cookie_upload_files(imgpaths=imglist,url=imgdata[host]['url'].strip('/'), cookie=imgdata[host]['cookie'], form=form)
        else:
            success=False
            logger.warning('图床'+host+'配置信息缺失')
        if res=='':
            success=False
    elif 'freeimage' in host.lower():
        host='freeimage'
        logger.info('正在尝试使用'+host+'上传图片,请稍等...')
        if imgdict[host]==True:
            res=chevereto_cookie_upload_files(imgpaths=imglist,url=imgdata[host]['url'].strip('/'), cookie=imgdata[host]['cookie'], form=form)
        else:
            success=False
            logger.warning('图床'+host+'配置信息缺失')
        if res=='':
            success=False
    elif 'chevereto_api' in host.lower():
        host='chevereto_api'
        logger.info('正在尝试使用'+host+'上传图片,请稍等...')
        if not (chevereto_url=='' or chevereto_apikey==''):
            res=chevereto_api_upload_files(imgpaths=imglist,url=chevereto_url,api_key=chevereto_apikey,form=form)
        else:
            success=False
            logger.warning('图床'+host+'配置信息缺失')
        if res=='':
            success=False
    elif 'chevereto_cookie' in host.lower():
        host='chevereto_cookie'
        logger.info('正在尝试使用'+host+'上传图片,请稍等...')
        if not (chevereto_url=='' or chevereto_cookie==''):
            res=chevereto_cookie_upload_files(imgpaths=imglist,url=chevereto_url, cookie=chevereto_cookie, form=form)
        else:
            success=False
            logger.warning('图床'+host+'配置信息缺失')
        if res=='':
            success=False
    else:
        host='ptpimg'
        logger.info('未找到该图床，正在尝试使用'+host+'上传图片,请稍等...')
        if imgdict[host]==True:
            res=ptpimg_upload_files(imgpaths=imglist,api_key=imgdata[host]['apikey'],form=form)
        else:
            success=False
            logger.warning('图床'+host+'配置信息缺失')
        if res=='':
            success=False
    if success:
        logger.info('上传成功！')
        return res
    else:
        logger.warning('上传失败！')
        if fail==False:
            return img_upload_seq(imgdata=imgdata,seq=seq,imglist=imglist,form=form)
        else:
            return ''



from loguru import logger
import os
from upload_machine.utils.para_ctrl.para_ctrl import *
from upload_machine.utils.site.site import makesites
from upload_machine.utils.pathinfo.pathinfo import findpathinfo
from upload_machine.utils.seed_machine.seed_machine import start_machine
from upload_machine.utils.img_upload.imgupload import img_upload
from upload_machine.utils.mediafile.mediafile import *
from doubaninfo.doubaninfo import getdoubaninfo

@logger.catch
def main():
    os.system('clear')
    logger.info("欢迎使用sauterne开发的Upload_Machine\n")
    yamlinfo=read_para()
    #设置路径，如果有下载文件都下载到screenshot_path
    os.chdir(yamlinfo['basic']['screenshot_path'])

    if 'basic' in yamlinfo and 'log' in yamlinfo['basic'] and yamlinfo['basic']['log']!=None:
        log = yamlinfo['basic']['log']
        logger.add(log, level="TRACE", backtrace=True, diagnose=True)

    if yamlinfo['mod']=='img_upload':
        logger.info('正在使用上传图床模式')
        res=img_upload(imgdata=yamlinfo['image hosting'],imglist=yamlinfo['imgfilelist'],host=yamlinfo['img_host'],form=yamlinfo['img_form'])
        logger.info('成功上传图床')
        print(res)

    if yamlinfo['mod']=='upload':

        sites=makesites(yamlinfo['site info'])
        #for item in sites:
        #    sites[item].print()
        
        pathlist=findpathinfo(yamlinfo,sites)
        #for item in pathlist:
        #    item.print()

        start_machine(pathlist,sites,yamlinfo)
        write_yaml(yamlinfo)

    if yamlinfo['mod']=='douban_info':
        #doubaninfo(yamlinfo['douban_url'])
        if 'doubancookie' in yamlinfo['basic'] and yamlinfo['basic']['doubancookie']:
            getdoubaninfo(url=yamlinfo['douban_url'],cookie=yamlinfo['basic']['doubancookie'])
        else:
            getdoubaninfo(url=yamlinfo['douban_url'])

    if yamlinfo['mod']=='media_img':
        screenshotnum=int(yamlinfo['basic']['picture_num'])
        screenshotaddress=yamlinfo['basic']['screenshot_path']
        takescreenshot(yamlinfo['media_file'],screenshotaddress,screenshotnum)
        imgpaths=[]
        for i in range (screenshotnum):
            imgpaths.append(os.path.join(screenshotaddress,str(i+1)+'.jpg'))
        res=img_upload(imgdata=yamlinfo['image hosting'],imglist=imgpaths,host=yamlinfo['img_host'],form=yamlinfo['img_form'])
        if res=='':
            print(self.chinesename+'上传图床失败')
        else:
            print('成功获得图片链接：\n'+res)

if __name__ == '__main__':
    main()


import os
import re
import random
from pathlib import Path
from typing import Optional, List
from string import ascii_letters, digits
import requests
from loguru import logger
from lxml.html import fromstring



def get_csrf_token(session) -> Optional[str]:
    req = session.get('https://imgbox.com')
    if req.ok:
        m = re.search(r"content=\"(.*?)\" name=\"csrf-token\"", req.text)
        if m:
            return m.groups()[0]
    return None


def get_token(session, csrf_token: str, gallery_title: Optional[str], allow_comment: bool = False) -> dict:
    headers = {
        'X-CSRF-Token': csrf_token,
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.99 Safari/537.36'
    }
    data = {
        "gallery": "true" if gallery_title is not None else "false",
        # TODO: need to test when gallery title is None
        "gallery_title": gallery_title,
        "comments_enabled": str(int(allow_comment)),
    }
    req = session.post("https://imgbox.com/ajax/token/generate", data=data, headers=headers, json=True)
    if req.ok and req.json().get("ok"):
        return req.json()
    return {}

def login(session, username, password, csrf_token):
    headers = {
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.99 Safari/537.36'
    }
    data = {
        "utf8": "✓",
        "authenticity_token": csrf_token,
        "user[login]": username,
        "user[password]": password
    }
    logger.info("正在登录imgbox...")
    req = session.post("https://imgbox.com/login", data=data, headers=headers)
    if len(req.history) and req.history[-1].status_code == 302:
        logger.info("登录成功")
        return
    logger.warning("登录失败，使用匿名模式上传")
        

def imgbox_upload_files(
    imgpaths: List[str],
    username: str = None,
    password: str = None,
    gallery_title: str = None,
    thumbnail_size: str = "300r",
    is_family_safe: bool = True,
    allow_comment: bool = False,
    form:str ="img",
) :
    for i in range(len(imgpaths)):
        imgpaths[i]=Path(imgpaths[i])
    session = requests.Session()
    csrf_token = get_csrf_token(session)
    if username and password:
        login(session, username, password, csrf_token)
    token = get_token(session, csrf_token, gallery_title, allow_comment)
    if not token:
        logger.warning("获取token失败")
        return []

    headers = {
        'X-CSRF-Token': csrf_token,
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.99 Safari/537.36'
    }
    urls = []
    for count, img in enumerate(imgpaths):
        data = {
            "token_id": str(token.get("token_id")),
            "token_secret": token.get("token_secret"),
            "content_type": str(int(is_family_safe)),
            "thumbnail_size": thumbnail_size,
            "gallery_id": token.get("gallery_id"),
            "gallery_secret": token.get("gallery_secret"),
            "comments_enabled": str(int(allow_comment)),
        }
        files = {
            "files[]": open(img, "rb"),
        }
        req = session.post(
            "https://imgbox.com/upload/process",
            data=data,
            files=files,
            headers=headers,
        )

        if not req.ok:
            logger.trace(req.content)
            logger.warning(f"上传图片失败: HTTP {req.status_code}, reason: {req.reason}")
            return None
        elif req.json().get("error"):
            logger.trace(req.content)
            logger.warning(
                f"上传图片失败: Code {req.json()['error'].get('code')}, message: {req.json()['error]'].get('message')}"
            )
            return None
        logger.info(f"第{count+1}张截图上传成功")
        urls.append(req.json())

    logger.info(f"Imgbox图床链接：https://imgbox.com/upload/edit/{token.get('token_id')}/{token.get('token_secret')}")
    urlstr=''
    for url in urls:
        if 'img' in form.lower():
            urlstr=urlstr+url.get("files", [{}])[0].get("original_url")+'\n'
        elif 'bbcode' in form.lower():
            urlstr=urlstr+'[url='+url.get("files", [{}])[0].get("original_url")+'][img]'+url.get("files", [{}])[0].get("thumbnail_url")+'[/img][/url]\n'
        else:
            urlstr=urlstr+url.get("files", [{}])[0].get("original_url")+'\n'
    return urlstr








class Torrent:
    def __init__(self):
        self.data={}
    def load(self,filename):
        self.stream=Stream(filename,"rb")
        if self.stream.next()==b'd':
            self.data=build_dic(self.stream)
        else:
            raise Exception("invalid torrent file")
    def dump(self,filename):
        self.stream=Stream(filename,"wb")
        dump_dic(self.stream,self.data)
        self.stream.close()
        
def build_dic(stream):#call after symbol 'd' found
    list=[]
    proxy_build(stream,list)
    dic={}
    for i in range(0,len(list)-1,2):   #convert a list[k1,v1,k2,v2...] to a dic
        dic[list[i]]=list[i+1]
    return dic
def build_list(stream):#call after symbol 'l' found
    list=[]
    proxy_build(stream,list)
    return list
def build_str(stream,str_len):#call after num found
    values=[]
    for i in range(str_len):
        values.append(stream.next())
    return b''.join(values)
def build_num(stream):#call after symbol 'i' found
    num=0
    minus=False
    while True:
        ch=stream.next()
        if ch>=b'0' and ch<=b'9':
            num=num*10+int(ch)
        elif ch==b'-':
            minus=True
        elif ch==b'e':
            break
        else:
            raise Exception("invalid torrent file")
    num=-num if minus else num
    return num
def proxy_build(stream,list): #because build_dic and build_list is simaliar,so I def a proxy-function  
    while True:
        ch=stream.next()
        if ch>=b'0' and ch<=b'9':#I think ch==0 is not very valid,but just do this.
            str_len=int(ch)
            while True:
                ch=stream.next()
                if ch==b':':
                    break
                elif ch>=b'0' and ch<=b'9':
                    str_len=str_len*10+int(ch)
                else:
                    raise Exception("invalid torrent file")
            list.append(build_str(stream,str_len))
        elif ch==b'l':
            list.append(build_list(stream))
        elif ch==b'i':
            list.append(build_num(stream))
        elif ch==b'd':
            list.append(build_dic(stream))
        elif ch==b'e':
            break
        else:
            raise Exception("invalid torrent file")

def str2bytes(s):
    return s.encode("utf-8")

def bytes2str(s):
    return s.decode("utf-8")
"""
for dumpfile
"""
import types
def dump_dic(stream,dic):
    stream.put(str2bytes("d"))
    for k,v in dic.items():
        dump_str(stream,k)
        proxy_dump(stream,v)
    stream.put(str2bytes("e"))
def dump_list(stream,list):
    stream.put(str2bytes("l"))
    for item in list:
        proxy_dump(stream,item)
    stream.put(str2bytes("e"))
def dump_str(stream,mystr):
    stream.put(str2bytes(str(len(mystr))+":"))
    stream.put(mystr)
def dump_num(stream,num):
    stream.put(str2bytes("i"))
    stream.put(str2bytes(str(num)))
    stream.put(str2bytes("e"))

def proxy_dump_python2(stream,item):
    if isinstance(item,types.DictionaryType):
        dump_dic(stream,item)
    elif isinstance(item,types.ListType):
        dump_list(stream,item)
    elif isinstance(item,types.StringType):
        dump_str(stream,item)
    elif isinstance(item,types.IntType) or isinstance(item,types.LongType):
        dump_num(stream,item)
    else:
        raise Exception("dump info error")
def proxy_dump(stream,item):
    if isinstance(item,dict):
        dump_dic(stream,item)
    elif isinstance(item,list):
        dump_list(stream,item)
    elif isinstance(item,bytes):
        dump_str(stream,item)
    elif isinstance(item,int):
        dump_num(stream,item)
    else:
        raise Exception("dump info error")
"""
byte stream
"""
class Stream:       
    def __init__(self,filename,mode):
        self.file=open(filename,mode)
    def next(self): #for mode r
        return self.file.read(1)
    def put(self,byte): #for mode w
        self.file.write(byte)
    def close(self):
        self.file.close()
    def __del__(self):
        self.file.close()
